@@ -44,7 +44,7 @@ class SST1RSoXSDB:
         '''
         if corr_mode == None:
             warnings.warn("Correction mode was not set, not performing *any* intensity corrections.  Are you sure this is "+
-                          "right? Set corr_mode to 'none' to suppress this warning.")
+                          "right? Set corr_mode to 'none' to suppress this warning.",stacklevel=2)
             self.corr_mode = 'none'
         else:
             self.corr_mode = corr_mode
@@ -103,15 +103,27 @@ class SST1RSoXSDB:
 
         
 
-        data = run['primary']['data'][md['detector']+'_image'] 
+        #data = run['primary']['data'][md['detector']+'_image'] 
+        #if self.dark_subtract:
+        #    dark = run['dark']['data'][md['detector']+'_image'].mean('time') #@TODO: change to correct dark indexing
+        #    image = data - dark - self.dark_pedestal
+        #else:
+        #    image = data - self.dark_pedestal
+            
+        data = run['primary']['data'][md['detector']+'_image'].astype(int) # convert from uint to handle dark subtraction
+        
         if self.dark_subtract:
-            dark = run['dark']['data'][md['detector']+'_image'].mean('time') #@TODO: change to correct dark indexing
-            image = data - dark - self.dark_pedestal
-        else:
-            image = data - self.dark_pedestal
+            dark = run['dark']['data'][md['detector']+'_image']
+            darkframe = np.copy(data.time)
+            for n,time in enumerate(dark.time):
+                darkframe[(data.time - time)>0]=int(n)
+            data = data.assign_coords(dark_id=("time", darkframe))
+            def subtract_dark(img,pedestal=100,darks=None):
+                return img + pedestal - darks[int(img.dark_id.values)]
+            data = data.groupby('time').map(subtract_dark,darks=dark,pedestal=self.dark_pedestal)
 
         if self.corr_mode != 'none':
-            warnings.warn('corrections other than none are not supported at the moment')
+            warnings.warn('corrections other than none are not supported at the moment',stacklevel=2)
 
 
         dims_to_join = []
@@ -120,7 +132,7 @@ class SST1RSoXSDB:
         for dim in dims:
             try:
                 test = len(md[dim])
-                dims_to_join.append(md[dim].data)
+                dims_to_join.append(md[dim])
                 dim_names_to_join.append(dim)
             except TypeError:
                 dims_to_join.append(np.ones(run.start['num_points'])*md[dim])
@@ -134,7 +146,7 @@ class SST1RSoXSDB:
                 dims_to_join,
                 names=dim_names_to_join)
 
-        retxr = image.squeeze('dim_0').rename({'dim_1':'pix_y','dim_2':'pix_x'}).assign_coords(time=index).rename({'time':'system'})#,md['detector']+'_image':'intensity'})
+        retxr = data.squeeze('dim_0').rename({'dim_1':'pix_y','dim_2':'pix_x'}).rename({'time':'system'}).assign_coords(system=index)#,md['detector']+'_image':'intensity'})
         
         #this is needed for holoviews compatibility, hopefully does not break other features.
         retxr = retxr.assign_coords({'pix_x':np.arange(0,len(retxr.pix_x)),'pix_y':np.arange(0,len(retxr.pix_y))})
@@ -190,14 +202,14 @@ class SST1RSoXSDB:
 
         else:
             md['rsoxs_config'] == 'unknown'
-            warnings.warn(f'RSoXS_Config is neither SAXS or WAXS. Looks to be {start["RSoXS_Config"]}.  Might want to check that out.')
+            warnings.warn(f'RSoXS_Config is neither SAXS or WAXS. Looks to be {start["RSoXS_Config"]}.  Might want to check that out.',stacklevel=2)
 
         if md['rsoxs_config']=='saxs':
             md['detector'] = 'Small Angle CCD Detector'
         elif md['rsoxs_config']=='waxs':
             md['detector'] = 'Wide Angle CCD Detector'
         else:
-            warnings.warn(f'Cannot auto-hint detector type without RSoXS config.')
+            warnings.warn(f'Cannot auto-hint detector type without RSoXS config.',stacklevel=2)
 
 
         # items coming from baseline  
@@ -213,20 +225,21 @@ class SST1RSoXSDB:
             'sam_th':'RSoXS Sample Rotation',
             'energy':'en_energy_setpoint',
             'polarization':'en_polarization_setpoint',
-            'exposure':md['detector']+'_cam_acquire_time'
+            'exposure':'RSoXS Shutter Opening Time (ms)' #md['detector']+'_cam_acquire_time'
         }
 
         for phs,rsoxs in md_lookup.items():
             try:
-                md[phs] = primary[rsoxs]
+                md[phs] = primary[rsoxs].values
+                #print(f'Loading from primary: {phs}, value {primary[rsoxs].values}')
             except KeyError:
                 try:
                     blval = baseline[rsoxs]
                     md[phs] = blval.mean().data
                     if blval.var() > 0:
-                        warnings.warn(f'While loading {rsoxs} to infill metadata entry for {phs}, found beginning and end values unequal: {baseline[rsoxs]}.  It is possible something is messed up.')
+                        warnings.warn(f'While loading {rsoxs} to infill metadata entry for {phs}, found beginning and end values unequal: {baseline[rsoxs]}.  It is possible something is messed up.',stacklevel=2)
                 except KeyError:
-                    warnings.warn(f'Could not find {rsoxs} in either baseline or primary.  Needed to infill value {phs}.  Setting to None.')
+                    warnings.warn(f'Could not find {rsoxs} in either baseline or primary.  Needed to infill value {phs}.  Setting to None.',stacklevel=2)
                     md[phs] = None
                     
         md['wavelength'] = 1.239842e-6 / md['energy']
@@ -281,7 +294,7 @@ class SST1RSoXSDB:
             corr = 1
 
         if(corr<0):
-            warnings.warn(f'Correction value is negative: {corr} with headers {headerdict}.')
+            warnings.warn(f'Correction value is negative: {corr} with headers {headerdict}.',stacklevel=2)
             corr = abs(corr)
 
 
@@ -289,7 +302,7 @@ class SST1RSoXSDB:
         # try:
         #     darkimg = self.darks[headerdict['EXPOSURE']]
         # except KeyError:
-        #     warnings.warn(f"Could not find a dark image with exposure time {headerdict['EXPOSURE']}.  Using zeros.")
+        #     warnings.warn(f"Could not find a dark image with exposure time {headerdict['EXPOSURE']}.  Using zeros.",stacklevel=2)
         #     darkimg = np.zeros_like(img)
 
         # img = (img-darkimg+self.dark_pedestal)/corr
