@@ -21,24 +21,29 @@ class SST1RSoXSLoader(FileLoader):
     pix_size_1 = 0.06
     pix_size_2 = 0.06
 
-    def __init__(self,corr_mode=None,user_corr_fun=None,dark_pedestal=0,exposure_offset=0):
+    def __init__(self,corr_mode=None,user_corr_func=None,dark_pedestal=100,exposure_offset=0,constant_md={}):
         '''
         Args:
             corr_mode (str): origin to use for the intensity correction.  Can be 'expt','i0','expt+i0','user_func','old',or 'none'
             user_corr_func (callable): takes the header dictionary and returns the value of the correction.
-            dark_pedestal (numeric): value to add to the whole image before doing dark subtraction, to avoid non-negative values.
+            dark_pedestal (numeric): value to subtract(/add, if negative) to the whole image.  this should match the instrument setting for suitcased tiffs, typically 100.
             exposure_offset (numeric): value to add to the exposure time.  Measured at 2ms with the piezo shutter in Dec 2019 by Jacob Thelen, NIST
+            constant_md (dict): values to insert into every metadata load. 
         '''
 
         if corr_mode == None:
             warnings.warn("Correction mode was not set, not performing *any* intensity corrections.  Are you sure this is "+
-                          "right? Set corr_mode to 'none' to suppress this warning.")
+                          "right? Set corr_mode to 'none' to suppress this warning.",stacklevel=2)
             self.corr_mode = 'none'
         else:
             self.corr_mode = corr_mode
-        # self.dark_pedestal = dark_pedestal
-        # self.user_corr_func = user_corr_func
-        # self.exposure_offset = exposure_offset
+
+
+        self.constant_md = constant_md
+
+        self.dark_pedestal = dark_pedestal
+        self.user_corr_func = user_corr_func
+        self.exposure_offset = exposure_offset
         # self.darks = {}
     # def loadFileSeries(self,basepath):
     #     try:
@@ -58,6 +63,17 @@ class SST1RSoXSLoader(FileLoader):
 
 
     def loadSingleImage(self,filepath,coords=None, return_q=False):
+        '''
+        HELPER FUNCTION that loads a single image and returns an xarray with either pix_x / pix_y dimensions (if return_q == False) or qx / qy (if return_q == True)
+
+
+        Args:
+            filepath (Pathlib.path): path of the file to load
+            coords (dict-like): coordinate values to inject into the metadata
+            return_q (bool): return qx / qy coords.  If false, returns pixel coords.
+
+        '''
+        
         img = Image.open(filepath)
 
         headerdict = self.loadMd(filepath)
@@ -84,26 +100,21 @@ class SST1RSoXSLoader(FileLoader):
             corr = 1
 
         if(corr<0):
-            warnings.warn(f'Correction value is negative: {corr} with headers {headerdict}.')
+            warnings.warn(f'Correction value is negative: {corr} with headers {headerdict}.',stacklevel=2)
             corr = abs(corr)
 
 
         # # step 2: dark subtraction
-        # try:
-        #     darkimg = self.darks[headerdict['EXPOSURE']]
-        # except KeyError:
-        #     warnings.warn(f"Could not find a dark image with exposure time {headerdict['EXPOSURE']}.  Using zeros.")
-        #     darkimg = np.zeros_like(img)
-
-        # img = (img-darkimg+self.dark_pedestal)/corr
+        # this is already done in the suitcase, but we offer the option to add/subtract a pedestal.
+        image_data = (np.array(img)-self.dark_pedestal)/corr
         if return_q:
             qpx = 2*np.pi*60e-6/(headerdict['sdd']/1000)/(headerdict['wavelength']*1e10)
-            qx = (np.arange(1,img.size[0]+1)-headerdict['beamcenter_x'])*qpx
-            qy = (np.arange(1,img.size[1]+1)-headerdict['beamcenter_y'])*qpx
+            qx = (np.arange(1,img.size[0]+1)-headerdict['beamcenter_y'])*qpx
+            qy = (np.arange(1,img.size[1]+1)-headerdict['beamcenter_x'])*qpx
             # now, match up the dims and coords
-            return xr.DataArray(img,dims=['qy','qx'],coords={'qy':qy,'qx':qx},attrs=headerdict)
+            return xr.DataArray(image_data,dims=['qy','qx'],coords={'qy':qy,'qx':qx},attrs=headerdict)
         else:
-            return xr.DataArray(img,dims=['pix_x','pix_y'],attrs=headerdict)
+            return xr.DataArray(image_data,dims=['pix_x','pix_y'],attrs=headerdict)
 
     def read_json(self,jsonfile):
         json_dict = {}
@@ -140,16 +151,16 @@ class SST1RSoXSLoader(FileLoader):
 
         else:
             json_dict['rsoxs_config'] == 'unknown'
-            warnings.warn('RSoXS_Config is neither SAXS or WAXS. Check json file')
+            warnings.warn('RSoXS_Config is neither SAXS or WAXS. Check json file',stacklevel=2)
 
         if json_dict['sdd'] == None:
-            warnings.warn('sdd is None, reverting to default values. Check json file')
+            warnings.warn('sdd is None, reverting to default values. Check json file',stacklevel=2)
             if json_dict['rsoxs_config'] == 'waxs':
                 json_dict['sdd'] = 38.745
             elif json_dict['rsoxs_config'] == 'saxs':
                 json_dict['sdd'] = 512.12
         if json_dict['beamcenter_x'] == None:
-            warnings.warn('beamcenter_x/y is None, reverting to default values. Check json file')
+            warnings.warn('beamcenter_x/y is None, reverting to default values. Check json file',stacklevel=2)
             if json_dict['rsoxs_config'] == 'waxs':
                 json_dict['beamcenter_x'] = 400.46
                 json_dict['beamcenter_y'] = 530.99
@@ -172,11 +183,11 @@ class SST1RSoXSLoader(FileLoader):
         primary_dict = {}
         df_primary = pd.read_csv(primary_csv)
         if json_dict['rsoxs_config'] == 'waxs':
-            primary_dict['exposure'] = df_primary['Wide Angle CCD Detector_cam_acquire_time'][seq_num]
+            primary_dict['exposure'] = df_primary['RSoXS Shutter Opening Time (ms)'][seq_num]
         elif json_dict['rsoxs_config'] == 'saxs':
-            primary_dict['exposure'] = df_primary['Small Angle CCD Detector_cam_acquire_time'][seq_num]
+            primary_dict['exposure'] = df_primary['RSoXS Shutter Opening Time (ms)'][seq_num]
         else:
-            warnings.warn('Check rsoxs_config in json file')
+            warnings.warn('Check rsoxs_config in json file',stacklevel=2)
 
         primary_dict['energy'] = round(df_primary['en_energy_setpoint'][seq_num],4)
         primary_dict['polarization'] = df_primary['en_polarization_setpoint'][seq_num]
@@ -238,6 +249,7 @@ class SST1RSoXSLoader(FileLoader):
         headerdict['rot2'] = 0
         headerdict['rot3'] = 0
 
+        headerdict.update(self.constant_md)
         return headerdict
 
     def peekAtMd(self,filepath):
